﻿/****************************************************
 * Prime.cs
 *
 *  A prime number generator based on Miller-Rabin's 
 *  randomized prime algorithm.
 * 
 * Uses Fermat's little Theorem:
 * 
 * If n is prime, then for all 1 <= a < n: 
 *      a^(n-1) (mod) n = 1
 * 
 * (c) 2008 by Tilman Mehler
 *
 *****************************************************/


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Prime
{
    static class Program
    {
        /// <summary>
        /// Recursively computes (uBase^uPower) mod uDivisor .
        /// </summary>

        static ulong RecursiveModulo(ulong uBase, ulong uPower, ulong uDivisor)
        {
            ulong uAux;
            
            if (uPower == 0) return 1;
            
            //a^2c mod n = (ac mod n)2 mod n
            else if (uPower % 2 == 0)
            {
                uAux = RecursiveModulo(uBase, uPower / 2, uDivisor);
                return (uAux * uAux) % uDivisor;
            }
            //ac+1 mod n = (ac mod n) · a mod n
            else return (RecursiveModulo(uBase , uPower - 1, uDivisor) * uBase) % uDivisor;
        }
        /// <summary>
        /// Tests, if the given number is prime by iterating the randomized prime test.
        /// </summary>
        /// <param name="uNumber">The number to test.</param>      
        /// <param name="uIterations">The number iterations.</param>      

        public static bool TestPrime(ulong uNumber, uint uIterations)
        {
            Random Rnd = new Random();
            ulong uAux;
            for (ulong i = 0; i < uIterations; ++i)
            {
                uAux = (ulong) (Rnd.Next()) % uNumber;
                if (uAux == 0) uAux = uNumber - 1;
                if (RecursiveModulo(uAux, uNumber - 1, uNumber) != 1)
                {
                    return false;
                }
            }
            return true;
        }

        static string s_sHelp =
            @"Prime - a prime number generator based on Miller-Rabins randomized prime test. "
            + Environment.NewLine
            + "(C) 2008 Tilman Mehler. " + Environment.NewLine + Environment.NewLine
            + "Usage: Prime.exe <r-min> <r-max> <i=10> - "
            + "computes all prime numbers int the range from r-min to r-max using i iterations.";

        public static void Main(string[] args)
        {
            ulong uMin, uMax;
            uint uIterations = 10;

            if (args.Length < 2 || args.Length > 3)
            {
                Console.WriteLine(s_sHelp);
                return;
            }
            try
            {
                uMin = ulong.Parse(args[0]);
                uMax = ulong.Parse(args[1]);
                if (args.Length == 3)
                {
                    uIterations = uint.Parse(args[2]);
                    if (uIterations < 1)
                    {
                        Console.WriteLine("Number of iterations must be >=1");
                        return;
                    }
                }
            }
            catch (Exception e)
            {
                System.Console.WriteLine(e.Message);
                return;
            }            
            System.Console.WriteLine("Determining Prime numbers between " +  uMin + " and " + uMax + " using " + uIterations + " iterations." );
            for (ulong i = uMin; i <= uMax; ++i)
            {
                if (TestPrime(i, uIterations)) Console.WriteLine(i);
            }
        }
    }
}
